/**
 * Klasse zum Lsen des n-Damen-Problems
 *
 * @author Daniel Garmann
 * @version 1.0 vom 22.09.2008
 */
public class Damen {
  private int damenanzahl = 8;       // damenanzahl
  private boolean[] zeileBedroht;    // zeile des brettes
  private boolean[] aufDiagBedroht;  // Aufwrtsdiagonalen
  private boolean[] abDiagBedroht;   // Abwrtsdiagonalen
  private int[] damen;               // merken, wo die Dame jeder Spalte steht
  private int loesungsAnzahl = 0;
  
  private String loesungen = "";     // Hier wird die Rckgabe gesammelt.

  public Damen(int anzahl) {
    damenanzahl = anzahl;
    zeileBedroht = new boolean[damenanzahl];
    aufDiagBedroht = new boolean[2 * damenanzahl-1];
    //Es wird festgehalten, ob in einer steigenden Diagonale schon eine
    //Dame steht. Fr alle Positionen einer solchen Dame gilt, dass zeile+spalte
    //konstant ist. Betrachtet man den Term zeile+spalte, dann
    //bewegt sich der index zwischen 0 und 2*damenzahl-1
    abDiagBedroht = new boolean[2 * damenanzahl-1];
    //Es wird festgehalten, ob in einer bestimmten fallenden 
    //Diagonalen schon eine Dame steht. Fr alle Positionen einer solchen
    //Diagonalen gilt, dass spalte-zeile konstant ist. Der Index bewegt sich
    //zwischen -(damenanzahl-1)  und damenanzahl-1, weshalb stets spalte-zeile + damenanzahl-1 gerechnet wird.
    damen = new int[damenanzahl];

    // Erst mal alles auf false, d.h. alles frei
    for (int i = 0; i < damenanzahl; i++)
      zeileBedroht[i] = false;

    for (int i = 0; i < (2 * damenanzahl) - 1; i++)
      aufDiagBedroht[i] = false;

    for (int i = 0; i < (2 * damenanzahl) - 1; i++)
      abDiagBedroht[i] = false;

    for (int i = 0; i < damenanzahl; i++)
      damen[i] = 0;
  }
  
  public Damen() {
    this(8);
  }

  private void setzeDame(int x, int y) {
    zeileBedroht[x] = true;
    aufDiagBedroht[x+y] = true;
    abDiagBedroht[x-y+damenanzahl-1] = true;
    damen[y] = x;
  }

  private void loescheDame(int x, int y) {
    zeileBedroht[x] = false;
    aufDiagBedroht[x+y] = false;
    abDiagBedroht[x-y+damenanzahl-1] = false;
    damen[y] = 0;
  }

  private boolean bedroht(int x, int y) {
    return (zeileBedroht[x] || aufDiagBedroht[x+y] ||
           abDiagBedroht[x - y+damenanzahl-1]);
  }

  private void dameSetzen(int spalte) {
    for (int zeile = 0; zeile < damenanzahl; zeile++) {
      if (!bedroht(zeile, spalte)) {
        setzeDame(zeile, spalte);

        if (spalte < damenanzahl-1) {
          dameSetzen(spalte + 1);
        } else {
          loesungMerken();
        }

        loescheDame(zeile, spalte);
      }
    }
  }

  private void loesungMerken() {
    String s = "";
    for (int spalte = 0; spalte < damenanzahl; spalte++) {
      s = s + damen[spalte];
      if (spalte < damenanzahl-1)
        s = s + "-";
    }
    loesungen = loesungen + s+"\n";
    loesungsAnzahl++;
  }

  public String startDame() {
    dameSetzen(0);
    loesungen = "Es ergaben sich " + loesungsAnzahl + " Loesungen\n"+
                "-------------------------------------------\n" + loesungen;
    return loesungen;
  }
}
